package term;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Enumeration;

/**
 * Termbaum of a ( 1 / f(x) ) Term.
 */
class Kehrwert extends TermBaum
{

	/** Derivation of a Function (1/f) is ( -(f')*1/(f*f) )
	 */
	public TermBaum ableitung(String variable)
	{
		Liste F = new Liste();
		F.addLast( getOperand().ableitung(variable) );

		Liste N = new Liste();
		N.addLast(getOperand());
		N.addLast(getOperand());

		F.addLast( new Kehrwert( new Produkt( N ) ) );

		return new Negativ( new Produkt( F ) );
	} // ableitung()
	/** Calculates the function value */
	public BigDecimal berechne(VariableTable Variablen, int Genauigkeit)
	{
		debugC("calculating 1 / ("+getOperand().toString()+")");
		BigDecimal Wert = getOperand().berechne(Variablen, Genauigkeit);

		try
		{
			return BigDecimal.valueOf(1).divide(Wert,Genauigkeit,BigDecimal.ROUND_HALF_UP);
		}
		catch (ArithmeticException e)
		{
			throw new TermFaultException("Error calculating 1/("+Wert.toString()+")");
		}
	} // berechne()
	/** Returns a copy of this object. */
	public TermBaum cloneTerm()
	{
		return new Kehrwert( getOperand() );
	}
/* Constructor parses a String into a TermBaum.
 */
public Kehrwert(String Term, int Offset)
{
	debugP("generating new Kehrwert");
	operand = parseTerm(Term, Offset);
} // Kehrwert()    
	// protected Constructor for clone()
	protected Kehrwert(TermBaum restterm)
	{
		operand = restterm;
	}
/** Adds '1/' to the rest of the Term. */
public String toString()
{
	if ( (getOperand() instanceof Konstante) ||
			 (getOperand() instanceof Funktion) ||
			 (getOperand() instanceof Variable) )
		return "1/" + getOperand().toString();
	else
		return "1/(" + getOperand().toString() + ")";
}    
	/** Simplifies the Term. */
	public TermBaum vereinfache()
	{
		TermBaum V = getOperand().vereinfache();

		if (V instanceof Kehrwert) //    1/(1/a) = a
			return ((Kehrwert)V).getOperand();

		boolean negativ;
		if (V instanceof Negativ)
		{
			//   (1/-a) = -(1/a)
			negativ=true;
			V = ((Negativ)V).getOperand();
		}
		else
			negativ=false;

		if (V instanceof Konstante && ((Konstante)V).isOne() )
		{
			if (negativ) //   1/(-1) = -1
				return new Negativ(V);
			else         //   1/1 = 1
				return V;
		} // if

		if (negativ)
			return new Negativ( new Kehrwert(V) );
		else
			return new Kehrwert(V);
	} // vereinfache()
} // class Kehrwert
