package term;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Enumeration;

/** --------------------------------------------------------------------------
 * Implementierung der ln-Funktion (natuerlicher Logarithmus)
 */
class LN extends Funktion
{
	/** Derivation of ln(a) is a'/a */
	public TermBaum ableitung(String variable)
	{
		Liste F = new Liste();
		F.addLast( new Kehrwert( getOperand() ) ); // aeussere mal
		F.addElement( getOperand().ableitung(variable) ); // innere Ableitung

		return (new Produkt(F)).vereinfache();
	} // ableitung()
	/** ln-Funktion erwartet genau einen Parameter */
	public final int AnzOperanden(){ return 1; }
	/** Anwendung der ln-Funktion auf den Funktionsparameter
	 */
	public BigDecimal berechne(VariableTable Variablen, int Genauigkeit)
	{
		debugC("calculating LN("+getOperand().toString()+")");
		double D = getOperand().berechne(Variablen, Genauigkeit).doubleValue();

		try
		{
			return new BigDecimal(Math.log(D)).setScale(Genauigkeit,
												BigDecimal.ROUND_HALF_UP);
		}
		catch (NumberFormatException e)
		{
			throw new TermFaultException("Error calculating LN("+D+")");
		}
	} // berechne()
	/** protected Constructor nur fuers package */
	protected LN(TermBaum T)
	{
		super(T);
	} // LN()
	/** Constructor fuer 'Funktion' */
	protected LN(){}
	// Das Vereinfachen nur zu dem Funktionsparameter durchreichen.
	public TermBaum vereinfache()
	{
		TermBaum T = getOperand().vereinfache();
		if (T instanceof EXP)
		{
			// LN(EXP(a)) = a 
			return ((EXP)T).getOperand();
		}
		else
		{
			if (T instanceof Kehrwert)
			{
				// LN(1/a) = -LN(a)
				return new Negativ( new LN( ((Kehrwert)T).getOperand() ) );
			}
			else
				return new LN(T);
		}
	} // vereinfache()
} // class LN
