package term;

import java.util.Enumeration;

/** Implementation of a simple linked list.
 * Allows fast appending of elements at the beginning and at the end
 * of the list.
 */
public class Liste
{
	/* erstes Element in der Liste (Referenz auf Elementkette)
	 * beginnt mit Dummy-Element !
	 */
	private ListEl first=null;
	private ListEl last =null;

	// Groesze der Liste
	private int Size=0;

	/** Class to store single elements in the linked list. */
	private class ListEl
	{
		public Object wert; // enthaltener Wert
		public ListEl next; // naechstes Element der Liste

		// verschiedene Konstruktoren
		public ListEl(ListEl Next, Object Elem)
		{
		  wert=Elem; next=Next;
		}
		public ListEl(Object E){ this(null,E); }
		public ListEl(){ this(null,null); }
	} // ListEl


	/** Enumeration of the list elements. The list will not be copied ! */
	private class ListEnumerator implements Enumeration
	{
		// Referenz zum naechsten Element der Aufzaehlung
		protected ListEl nextEl;

		// Constructor uebernimmt Referenz auf die Listenelemente
		public ListEnumerator(ListEl First)
		{
			nextEl = First;
		}

		public boolean hasMoreElements()
		{
			return (nextEl.next != null);
		}

		public Object nextElement()
		{
			nextEl = nextEl.next;
			return nextEl.wert;
		}
	} // ListEnumerator

	// Fuegt ein neues Element am Anfang ein
	public void addElement(Object E)
	{
		first.next = new ListEl(first.next, E);
		if (last==first)
			last=first.next;

		Size++;
	} // addElement()
	// Fuegt Element als letztes ein
	public void addLast(Object E)
	{
		last.next = new ListEl(E);
		last = last.next;

		Size++;
	} // addLast()
	// Erzeugen einer Kopie dieser Liste (Inhalt wird nicht mit kopiert !)
	public Object clone()
	{
		return new Liste(copyListEl(first));
	} // clone()
	/** generates a complete recursive copy of the element chain. */
	protected ListEl copyListEl(ListEl toCopy)
	{
		if (toCopy == null)
			return null;
		else
			return new ListEl( copyListEl(toCopy.next), toCopy.wert );
	} // copyListEl()
/**
 * Returns the element with the given index or <code>null</code>
 * if there is no element with that index.
 */
public Object elementAt(int index)
{
	if (index < 0  ||  index >= size())
		return null;

	ListEl temp = first.next;
	while (index-- > 0)
	{
		temp = temp.next;
	}

	return temp.wert;
}
	/** Liefert eine Enumeration der Elemente dieser Liste. Keine Kopie ! */
	public Enumeration elements()
	{
		return new ListEnumerator( first );
	} // elements()
	// liefert den Wert des ersten Listenelementes zurueck
	public Object firstEl()
	{
		return first.next.wert;
	}
	/** Liefert eine Enumeration einer Kopie der Elemente dieser Liste. */
	public Enumeration getElementCopy()
	{
		return new ListEnumerator( copyListEl(first) );
	} // elements()
	// ist die Liste leer ?
	public boolean isEmpty()
	{
		return first==last;
	}
	// protected Constructor fuegt Elementkette zu Objekt 'Liste' zusammen
	protected Liste(ListEl newFirst)
	{
		first = newFirst;
		last = first;

		while (last.next != null)
		{
			last = last.next;
			Size++;
		}
	} // Liste()    
	// Loeschen eines bestimmten Objektes aus der Liste
	public Object removeElement(Object E)
	{
		Object oldValue = null;
		
		ListEl actual = first;
		boolean weiter = true;

		do
		{
			if (E.equals(actual.next.wert))
			{
				if (actual.next==last)
					last = actual;

				oldValue = actual.next.wert;

				actual.next = actual.next.next;
				weiter = false;

				Size--;
			}
			else
				weiter = ((actual = actual.next).next != null);
		}
		while (weiter);

		return oldValue;
	} // removeElement()
	// Loeschen des ersten Elementes aus der Liste
	public Object removeFirst()
	{
		if (isEmpty())
			return null;
		else
		{
			Object value = first.next.wert;
			first.next = first.next.next;
			return value;
		}
	} // removeFirst()
// Groesze der Liste
public int size()
{
	return Size;
}    
  // public Constructor erzeugt neue leere Liste mit Dummy-Element
  public Liste()
  {
	first = new ListEl();
	last  = first;
  } // Liste()      
} // Liste
