package term;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Enumeration;

/** Implementierung der pot-Funktion (Potenzfunktion)
 */
class POT extends Funktion
{
	/** (a^b)' = (e^(b*ln(a)))' = e^(b*ln(a)) * (b*ln(a))' = (a^b) * (b*ln(a))'
	 */
	public TermBaum ableitung(String variable)
	{
		Liste F = new Liste();
		F.addLast( new LN( (TermBaum) Operanden.elementAt(0) ) ); // ln(a)
		F.addElement( Operanden.elementAt(1) ); // b

		TermBaum T = new Produkt(F).vereinfache(); // T = b*ln(a)

		F = new Liste();
		F.addLast( this ); // aeussere mal
		F.addLast( T.ableitung(variable) ); // innere Ableitung

		return new Produkt(F).vereinfache(); // (a^b) * T'
	} // ableitung()
  // pot-Funktion erwartet genau zwei Parameter
  protected final int AnzOperanden(){ return 2; }    
/** Applying the Powerfunction to the parameters: (a,b) -> a^b */
public BigDecimal berechne(VariableTable Variablen, int Genauigkeit)
{
	BigDecimal base = ((TermBaum) Operanden.elementAt(0)).berechne(Variablen, Genauigkeit),
			   exp  = ((TermBaum) Operanden.elementAt(1)).berechne(Variablen, Genauigkeit);

	debugC("calculating POT("+base.toString()+","+exp.toString()+")");

	try
	{
		exp.setScale(0,BigDecimal.ROUND_UNNECESSARY);
	}
	catch (ArithmeticException e)
	{
		throw new TermFaultException("POT-Function expects integer exponent");
	}

	try
	{
		base.setScale(0,BigDecimal.ROUND_UNNECESSARY);
	}
	catch (ArithmeticException e)
	{
		throw new TermFaultException("POT-Function expects integer base");
	}

	BigDecimal pot = new BigDecimal(base.toBigInteger().pow(exp.abs().intValue()));

	if (exp.compareTo(BigDecimal.valueOf(0)) == -1)
	{
		// Exponent < 0  =>  Ergebnis = 1/Ergebnis
		return BigDecimal.valueOf(1).divide(pot,Genauigkeit,BigDecimal.ROUND_HALF_UP);
	}
	else
		return pot.setScale(Genauigkeit,BigDecimal.ROUND_HALF_UP);
} // berechne()    
	// protected Constructor nur fuers package !
	protected POT(TermBaum B, TermBaum E)
	{
		Operanden = new Liste();
		Operanden.addLast(B);
		Operanden.addLast(E);
	} // POT()
  // Constructor fuer 'Funktion'
  protected POT(){}      
	// Das Vereinfachen nur zu den Funktionsparametern durchreichen.
	public TermBaum vereinfache()
	{
		TermBaum B = ((TermBaum) Operanden.elementAt(0)).vereinfache();
		TermBaum E = ((TermBaum) Operanden.elementAt(1)).vereinfache();

		return new POT(B,E);
	} // vereinfache();
} // class POT
