package term;

import java.math.BigDecimal;
import java.math.BigInteger;

import java.util.Enumeration;

/** Implementation of the power function
 */
class Power extends TermBaum
{
/** (a^b)' = (e^(b*ln(a)))' = e^(b*ln(a)) * (b*ln(a))' = (a^b) * (b*ln(a))'
 */
public TermBaum ableitung(String variable)
{
	Liste F = new Liste();
	F.addLast( new LN( (TermBaum) Operanden.elementAt(0) ) ); // ln(a)
	F.addElement(  Operanden.elementAt(1) );        // b

	TermBaum T = new Produkt(F).vereinfache(); // T = b*ln(a)

	F = new Liste();
	F.addLast( this ); // aeussere mal
	F.addLast( T.ableitung(variable) ); // innere Ableitung

	return new Produkt(F).vereinfache(); // (a^b) * T'
} // ableitung()
/** Applying the Powerfunction to the parameters: (a,b) -> a^b */
public BigDecimal berechne(VariableTable Variablen, int Genauigkeit)
{
	BigDecimal base = ((TermBaum) Operanden.elementAt(0)).berechne(Variablen, Genauigkeit),
			   exp  = ((TermBaum) Operanden.elementAt(1)).berechne(Variablen, Genauigkeit);

	debugC("calculating "+base.toString()+"^"+exp.toString());

	try
	{
		exp.setScale(0,BigDecimal.ROUND_UNNECESSARY);
	}
	catch (ArithmeticException e)
	{
		throw new TermFaultException("Power function expects integer exponent");
	}

	try
	{
		base.setScale(0,BigDecimal.ROUND_UNNECESSARY);
	}
	catch (ArithmeticException e)
	{
		throw new TermFaultException("Ppwer function expects integer base");
	}

	BigDecimal pot = new BigDecimal(base.toBigInteger().pow(exp.abs().intValue()));

	if (exp.compareTo(BigDecimal.valueOf(0)) == -1)
	{
		// Exponent < 0  =>  Ergebnis = 1/Ergebnis
		return BigDecimal.valueOf(1).divide(pot,Genauigkeit,BigDecimal.ROUND_HALF_UP);
	}
	else
		return pot.setScale(Genauigkeit,BigDecimal.ROUND_HALF_UP);
} // berechne()    
/** Returns a copy of this power function
 */
public TermBaum cloneTerm()
{
	Enumeration ops = Operanden.elements();
	return new Power( (TermBaum)ops.nextElement(), (TermBaum)ops.nextElement() );
}
	/** protected constructor to parse the TermBaum. */
	protected Power(String term, int offset)
	{
		Operanden = new Liste();

		BracketST ops = new BracketST(term, "^", "(", ")");
		String base, exp, token;
		int pos = 0; // offset of the last token
		int exponentStart = 0; // start position of the exponent part

		if (! ops.hasMoreTokens())
			throw new TermFaultException(offset, "Missing base for power function");

		token = ops.nextToken();

		if (token.equals("^"))
			throw new TermFaultException(offset+pos, "Displaced power symbol");

		// parse the base operand
		Operanden.addLast( parseTerm(token, offset) );

		pos = ops.getNextPos();
		if (! ops.hasMoreTokens())
			throw new TermFaultException(offset + pos, "Missing power function symbol");
		
		token = ops.nextToken();

		exponentStart = ops.getNextPos();
		if (! ops.hasMoreTokens())
			throw new TermFaultException(offset + exponentStart, "Missing exponent");

		// find the exponent
		StringBuffer exponent = new StringBuffer( term.length() - exponentStart );

		pos = ops.getNextPos();
		token = ops.nextToken();

		if (token.equals("^"))
			throw new TermFaultException(offset+pos, "Displaced power symbol");

		exponent.append(token);

		// convert multiple exponents into a product (^ => *) and parse them separately
		while (ops.hasMoreTokens())
		{
			pos = ops.getNextPos();
			if (ops.nextToken().equals("^"))
				exponent.append('*');
			else
				throw new TermFaultException(offset + pos, "Unexpected symbol");

			pos = ops.getNextPos();
			if (! ops.hasMoreTokens())
				throw new TermFaultException(offset + pos, "Missing exponent");

			token = ops.nextToken();

			if (token.equals("^"))
				throw new TermFaultException(offset+pos, "Displaced power symbol");

			exponent.append(token);
		}

		Operanden.addLast( parseTerm(exponent.toString(), offset) );
	}
	// protected Constructor nur fuers package !
	protected Power(TermBaum B, TermBaum E)
	{
		Operanden = new Liste();
		Operanden.addLast(B);
		Operanden.addLast(E);
	} // Power()
	/** Converts this term into a String representation.
	 */
	public String toString()
	{
		Enumeration ops = Operanden.elements();
		TermBaum base = (TermBaum) (ops.nextElement());
		TermBaum exp  = (TermBaum) (ops.nextElement());

		StringBuffer term = new StringBuffer();
		
		if ((base instanceof Konstante) || (base instanceof Variable))
			term.append(base);
		else
			term.append('(').append(base).append(')');

		term.append('^');

		if ((exp instanceof Konstante) || (exp instanceof Variable))
			term.append(exp);
		else
			term.append('(').append(exp).append(')');

		return term.toString();
	}
  // Constructor fuer 'Funktion'
  public Power(){}      
	// Das Vereinfachen nur zu den Funktionsparametern durchreichen.
	public TermBaum vereinfache()
	{
		TermBaum B = ((TermBaum) Operanden.elementAt(0)).vereinfache();
		TermBaum E = ((TermBaum) Operanden.elementAt(1)).vereinfache();

		return new Power(B,E);
	} // vereinfache();
} // class Power
