package term;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Enumeration;

/**
 * Termbaum eines Produkts
 */
class Produkt extends TermBaum
{

	/** Ableitung eines Produkts (erweiterte Produktregel)
	 *	(a*b*c*..)' = a'*b*c*.. + a*b'*c*.. + a*b*c'*.. + ...
	 */
	public TermBaum ableitung(String variable)
	{
		Liste Sum = new Liste();

		Enumeration Faktoren = Operanden.elements();

		while (Faktoren.hasMoreElements())
		{
			Liste Summandenprodukt = new Liste();
			TermBaum actual = (TermBaum)Faktoren.nextElement();

			Enumeration E = Operanden.elements();
			while (E.hasMoreElements())
			{
				TermBaum F = (TermBaum) E.nextElement();

				if (F == actual)
					Summandenprodukt.addLast( F.ableitung(variable) );
				else
					Summandenprodukt.addLast( F );
			} // while

			Sum.addLast( new Produkt( Summandenprodukt ) );
		} // while

		return new Summe( Sum );
	} // ableitung()
	/** Operanden nacheinander berechnen, Ergebnis aufsummieren und zurueckliefern. */
	public BigDecimal berechne(VariableTable Variablen, int Genauigkeit)
	{
		debugC("calculating Product");

		// Produkt ist anfangs 1
		BigDecimal Prod = BigDecimal.valueOf(1).setScale(Genauigkeit, BigDecimal.ROUND_HALF_UP);
		Enumeration Ops = Operanden.elements();

		// Operanden der Reihe nach aufsummieren
		while (Ops.hasMoreElements())
		{
			Object T = Ops.nextElement();
			if (T instanceof Kehrwert)
			{
				// zu groszen Rundungsfehlern vorbeugen !
				BigDecimal F = ((Kehrwert)T).getOperand().berechne(Variablen, Genauigkeit);

				debugC("multiplying by 1/"+F.toString());

				try
				{
					Prod = Prod.divide(F, Genauigkeit, BigDecimal.ROUND_HALF_UP);
				}
				catch (ArithmeticException e)
				{
					throw new TermFaultException(
								"Error calculating 1/(" + F.toString() + ")");
				}
			} // if
			else
			{
				BigDecimal F = ((TermBaum)T).berechne(Variablen,Genauigkeit);
				debugC("multiplying by "+F.toString());
				Prod = Prod.multiply(F).setScale(Genauigkeit, BigDecimal.ROUND_HALF_UP);
			} // else

		} // while

		return Prod;
	} // berechne()
	// Liefert vollstaendige Kopie des TermBaums
	public TermBaum cloneTerm()
	{
		return new Produkt((Liste)Operanden.clone());
	}
	/** parst Produktterm in einzelne Operanden. */
	public Produkt(String Term, int Offset)
	{
		debugP("generating  Product");
		Operanden = new Liste();

		// immer Rechenzeichen ganz am Anfang => Anfang kein Sonderfall
		Term = "*" + Term;
		Offset--; // Offset fuer diesen Trick anpassen ('*' war nicht im Term !)

		// Term nach Multiplikations-/Divisionszeichen zerteilen
		BracketST ST = new BracketST(Term, "*/", "(", ")");
		String next;
		int pos=0; // Offset des jew. Operanden im Summenterm

		while (ST.hasMoreTokens())
		{
			// fetch the calculation symbol
			String Rz = ST.nextToken();

			pos = ST.getNextPos();
			if (!ST.hasMoreTokens())
				throw new TermFaultException(Offset + pos, "Missing Operand");

			// naechsten Operanden abtrennen und Offset darauf setzen
			pos = ST.getNextPos();
			next = ST.nextToken();
			
			if (next.length()==1 && (next.equals("/") || next.equals("*")))
				throw new TermFaultException(Offset + pos, "Operand expected");

			TermBaum T;
			if (Rz.equals("/")) //  a/b = a*(1/b)  =>  Quotient = Produkt;
				T = new Kehrwert(next, Offset + pos);
			else
				T = parseTerm(next,Offset + pos);

			Operanden.addLast(T);
		} // while

	} // Produkt()
	// Constructor zum Clonen
	protected Produkt(Liste Ops){ super(Ops); }
	// Umwandeln des Produktbaums in einen String
	public String toString()
	{
		Enumeration Ops = Operanden.elements();
		String Term = "";
		while (Ops.hasMoreElements())
		{
			TermBaum T = (TermBaum)Ops.nextElement();
			if (T instanceof Kehrwert)
			{
				// P*1/a = P/a
				if (Term.length()==0)
					Term += "1/";
				else
					Term += "/";

				T = ((Kehrwert)T).getOperand();
			}
			else if (Term.length()>0)
				Term += "*";

			if ((T instanceof Funktion)  ||
					(T instanceof Konstante) ||
					(T instanceof Variable))
				Term += T.toString();
			else
				Term += "(" + T.toString() + ")";
		} // while

		return Term;
	} // toString()
	
	/** Vereinfachen eines Produktbaums:
	 *  Vorzeichen sammeln, 1-Faktoren entfernen
	 *  geklammerte Produkt-Faktoren eine Ebene hoeher heben
	 */
	public TermBaum vereinfache()
	{
		Liste L = new Liste();
		Enumeration Faktoren = Operanden.elements();

		// Faktoren vereinfachen und Vorzeichen vor das Produkt ziehen
		boolean negativ=false;
		while (Faktoren.hasMoreElements())
		{
			TermBaum F = ((TermBaum)Faktoren.nextElement()).vereinfache();

			if (F instanceof Negativ)
			{
				negativ = !negativ;
				F = ((Negativ)F).getOperand();
			}

			if (F instanceof Produkt)
			{
				// Produkte koennen zusammengefasst werden: a*(b*c) = a*b*c
				Enumeration E = F.Operanden.elements();
				while (E.hasMoreElements())
					L.addLast(E.nextElement());
			}
			else if (F instanceof Konstante)
			{
				// ansonsten auf spezielle Faktoren untersuchen
				Konstante constant = (Konstante) F;

				if ( constant.isZero() )
				{
					// a*0*b = 0 = F
					return constant;
				}
				else if ( ! constant.isOne() )
				{
					// Einsen weglassen, andere Faktoren wieder anhaengen
					L.addLast(F);
				}
			}
			else
				L.addLast(F);
		} // while

		// verbleibende Operandenliste auswerten
		TermBaum V;

		if (L.size()==0)
		{
			// keine Faktoren mehr da => Produkt = +1 / -1
			V = new Konstante( BigDecimal.valueOf(1) );
		}
		else if (L.size()==1)
		{
			// nur ein Faktor uebrig =>  = Ergebnis
			V = (TermBaum)L.firstEl();
		}
		else
		{
			// mehrere Faktoren wieder zum Produkt machen
			V = new Produkt(L);
		}

		// Vorzeichen des Produkts ggf. hinzufuegen
		if (negativ)
			return new Negativ( V );
		else
			return V;
	} // vereinfache()
} // class Produkt
