package term;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Enumeration;

/**
 * Termbaum einer Summe
 */
class Summe extends TermBaum
{

	/** Ableitung einer Summe: (a+b+...)' = a' + b' + ...
	 */
	public TermBaum ableitung(String variable)
	{
		Liste S = new Liste();
		Enumeration E = Operanden.elements();

		while (E.hasMoreElements())
			S.addLast( ((TermBaum) E.nextElement()).ableitung(variable) );

		return new Summe(S);
	}
	/** Operanden nacheinander berechnen, Ergebnis aufsummieren
	 *  und zurueckliefern
	 */
	public BigDecimal berechne(VariableTable Variablen, int Genauigkeit)
	{
		debugC("calculating sum");

		// Summe ist anfangs 0
		BigDecimal Sum = BigDecimal.valueOf(0).setScale(Genauigkeit, BigDecimal.ROUND_HALF_UP);

		// Operanden der Reihe nach aufsummieren
		Enumeration Ops = Operanden.elements();

		while (Ops.hasMoreElements())
		{
			TermBaum op = (TermBaum) Ops.nextElement();

			if (op instanceof Negativ)
			{
				// unneccessary, but a little faster ...
				BigDecimal summand = op.getOperand().berechne(Variablen,Genauigkeit); 
				debugC( "subtracting " + summand.toString() );
				Sum = Sum.subtract( summand );
			}
			else
			{
				BigDecimal summand = op.berechne(Variablen,Genauigkeit);
				debugC( "adding " + summand.toString() );
				Sum = Sum.add( summand );
			}
		}

		return Sum;
	} // berechne()
	// Liefert vollstaendige Kopie des TermBaums
	public TermBaum cloneTerm()
	{
		return new Summe((Liste)Operanden.clone());
	}
	/** parst Summenterm in einzelne Operanden */
	public Summe(String Term, int Offset)
	{
		debugP("generating sum");
		Operanden = new Liste();

		// immer Rechenzeichen ganz am Anfang => Anfang kein Sonderfall
		if (Term.charAt(0) != '-' && Term.charAt(0) != '+')
		{
			Term = "+" + Term;
			Offset--; // Offset fuer diesen Trick anpassen ('+' war nicht im Term !)
		} // if

		// Term nach Plus-/Minuszeichen zerteilen
		BracketST ST = new BracketST(Term, "+-", "(", ")");
		String next;
		int pos=0; // Offset des jew. Operanden im Summenterm

		while (ST.hasMoreTokens())
		{
			// erstes Rechenzeichen holen
			String Rz = ST.nextToken();

			if (!ST.hasMoreTokens())
				throw new TermFaultException(Offset + ST.getNextPos(), "Missing Operand");

			// naechsten Operanden abtrennen und Offset darauf setzen
			pos = ST.getNextPos();
			next = ST.nextToken();

			if (next.equals("+") || next.equals("-"))
			{
				// strict syntax forbids "a+-b" = a+(-b)
				if ( StrictSyntax || (! ST.hasMoreTokens()) )
					throw new TermFaultException(Offset + pos, "Operand expected");
				else
					next += ST.nextToken();
			}

			TermBaum T;
			if (Rz.equals("-")) // - = +-  => (Differenz = Summe);
				T = new Negativ(next, Offset + pos);
			else
				T = parseTerm(next, Offset + pos);

			Operanden.addLast(T);
		} // while

	} // Summe()
	// Constructor zum Clonen und fuer "Produkt.ableitung()"
	protected Summe(Liste Ops)
	{
		super(Ops);
	}
	public String toString()
	{
		Enumeration Ops = Operanden.elements();
		String Term = Ops.nextElement().toString();

		while (Ops.hasMoreElements())
		{
			TermBaum T = (TermBaum)Ops.nextElement();
			if (T instanceof Negativ) // statt +- reicht -
				Term += T.toString();
			else
				Term += "+" + T.toString();
		}
		return Term;
	} // toString()    
	/** Vereinfachen eines Summenterms:
	 * Untersummen in diese Summe uebernehmen,
	 * Minimierung der Anzahl negativer Vorzeichen,
	 * Entfernung von 0-Summanden
	 */
	public TermBaum vereinfache()
	{
		Enumeration E = Operanden.elements();
		Liste S = new Liste(); // alle Summanden werden vereinfacht hierher kopiert

		// zuerst Summanden vereinfachen und Untersummen zusammenfassen
		while (E.hasMoreElements())
		{
			TermBaum Summand = ((TermBaum)E.nextElement()).vereinfache();
			if (Summand instanceof Summe)
			{
				// Summanden aus Untersummen direkt uebernehmen
				Enumeration Sum = ((Summe)Summand).Operanden.elements();
				while (Sum.hasMoreElements())
					S.addLast(Sum.nextElement());
			}
			else if (Summand instanceof Negativ
					&& ((Negativ)Summand).getOperand() instanceof Summe)
			{
				// auch -(Summe) ist eine Untersumme !
				Enumeration Sum = ((Negativ)Summand).getOperand().Operanden.elements();
				while (Sum.hasMoreElements())
					S.addLast( new Negativ( (TermBaum) Sum.nextElement() ) );
			}
			else if (! (Summand instanceof Konstante
						&& ((Konstante)Summand).isZero()) )
			{
				// 0-Summanden nicht uebernehmen
				S.addLast(Summand);
			} // else
		} // while


		if (S.size()<2)
		{
			// zu viele Summanden weggefallen um Summe zu ergeben
			if (S.size()==0) // alle Summanden waren 0 => Summe=0
				return new Konstante( BigDecimal.valueOf(0) );
			else // nur ein Summand uebrig  =>  = Summe
				return (TermBaum) S.firstEl();
		} // if


		// negative Vorzeichen zaehlen und Liste inverser Summanden bilden
		Liste Sm = new Liste(); // alle Summanden werden invertiert hierher kopiert
		int neg=0; // Anzahl der gefundenen negativen Summanden

		E = S.elements();
		while(E.hasMoreElements())
		{
			TermBaum Summand = ((TermBaum)E.nextElement());

			// Inverses des Summanden bestimmen
			TermBaum NegSum;
			if (Summand instanceof Negativ)
			{
				Sm.addLast( ((Negativ)Summand).getOperand() );
				neg++;
			}
			else
				Sm.addLast( new Negativ(Summand) );
		} // while

		if (neg>S.size()/2) // neg. Summanden mehr als pos. => -(-1*Summe)
			return new Negativ( new Summe(Sm) );
		else
			return new Summe(S);
	} // vereinfache()
} // class Summe
