package term;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Enumeration;

/** Representation of a variable inside a TermBaum.
 */
final class Variable extends TermBaum
{
	/** The name of the variable. */
	private String varName;
/** The derivation of a Variable with respect to itself is 1,
 *  otherwise it is 0.
 */
public TermBaum ableitung(String variable)
{
	if (variable.equals(varName))
		return new Konstante( BigDecimal.valueOf(1) );
	else
		return new Konstante( BigDecimal.valueOf(0) );
} // ableitung()    
/** Calculate the value of this Variable by looking it up in the VariableTable. */
public BigDecimal berechne(VariableTable Variablen, int Genauigkeit)
{
	debugC("calculating Variable "+varName);

	TermBaum value = Variablen.getValue(varName);

	if (value == null)
		throw new TermFaultException("Unknown Variable "+varName);
	else
		return value.berechne(Variablen, Genauigkeit);
} // berechne()
/** Build a copy of this variable */
public TermBaum cloneTerm(){ return new Variable(varName); }
public String getName()
{
	return varName;
}
/** Constructor to parse the name of a Variable */
public Variable(String Term, int Offset)
{
	debugP("generating Variable");
	if (Term.charAt(0) != '#')
		throw new TermFaultException(Offset,"Variables start with \'#\'");

	if ((Term.length()<2) || !Character.isLetter(Term.charAt(1)))
		throw new TermFaultException(Offset+1,
					"The first character of a variable name must be a letter");


	// the second character is a letter, then letter or digit
	for (int p=2; p<Term.length(); p++)
	{
		if (! Character.isLetterOrDigit(Term.charAt(p)) )
			throw new TermFaultException(Offset+p,
						"Variable names consist of letters and digits");
	} // for

	varName = Term;
} // Variable()
/** Constructor for clone() */
protected Variable(String Name){ varName = Name; }    
/** Returns the variable name. */
public String toString()
{
	return getName();
}
/** Variables cannot be simplified. */
public TermBaum vereinfache(){ return this.cloneTerm(); }    
} // class Variable
